'' =================================================================================================
''
''   File....... jm_dht11_ez.spin
''   Purpose.... Background temp/humidity measurement with DHT11 or DHT22 sensor
''   Author..... Jon "JonnyMac" McPhalen
''               Copyright (C) 2015 Jon McPhalen
''               -- see below for terms of use
''   E-mail..... jon@jonmcphalen.com
''   Started....
''   Updated.... 20 DEC 2015
''
'' =================================================================================================


con { fixed io pins }

  RX1 = 31                                                       ' programming / terminal
  TX1 = 30

  SDA = 29                                                       ' eeprom / i2c
  SCL = 28
  

var

  long  cog
  long  stack[32]

  long  rhtresult
  long  count
  long  badchecksum

  
pub start(pin, secs)

'' Start DHT11/22 cog
'' -- pin defines IO connection (pull up to 5v through 4.7K)
'' -- secs is sampling period, usually 1 or 2

  stop                                                           ' stop driver if running

  outa[pin] := 0                                                 ' clear pin in this cog  
  dira[pin] := 0                                                   

  cog := cognew(scan_dht11(pin, secs), @stack) + 1               ' start the cog

  return cog


pub stop

'' Stop DHT11 cog if running

  if (cog)                                                       ' running
    cogstop(cog - 1)                                             '  yes, stop it
    cog := 0                                                     '  mark stopped
    
  longfill(@rhtresult, 0, 3)                                     ' clear user variables


pub reading

'' Return current reading
'' -- new value = new rh/t results

  return count


pub error

'' Returns true if problem with checksum

  return badchecksum


pub rh

'' Return relative humidity (whole %)

  return rhtresult.byte[0]


pub rh10

'' Return relative humidity (tenths %)

  return (rhtresult.byte[0] * 10) + rhtresult.byte[1]    


pub tc

'' Return temperature in Celsius (whole degrees)

  return rhtresult.byte[2]  


pub tc10

'' Return temperature in 0.1 degree Celsius

  return (rhtresult.byte[2] * 10) + rhtresult.byte[3]    


pub tf

'' Return temperature in Fahrenheit (whole degrees) 

  return tc * 9 / 5 + 32


pub tf10

'' Return temperature in 0.1 degree Fahrenheit
  return tc10 * 9 / 5 + 32_0

  
pri scan_dht11(pin, secs) {                                      ' call only with cognew()
  } | mask, us1, bit1, t, idx, nbit, raw, cs  

  mask := |< pin                                                 ' create mask from pin
  secs := (1 #> secs <# 10) * clkfreq                            ' limit period, convert to ticks
  us1  := clkfreq / 1_000_000                                    ' ticks in 1us
  bit1 := us1 * 35                                               ' minimum threshold for 1 bit

  ctra := (%01000 << 26) | pin                                   ' measure positive pulse
  frqa := 1

  t := cnt
  repeat
    dira[pin] := 1                                               ' pull pin low
    waitcnt(t += (us1 * 20_000))                                 ' hold 20ms
    dira[pin] := 0                                               ' release pin
    longfill(@raw, 0, 2)                                         ' clear raw and cs     
    
    waitpne(mask, mask, 0)                                       ' response low
    waitpeq(mask, mask, 0)                                       ' response high
    waitpne(mask, mask, 0)                                       ' start of packet
    
    repeat idx from 0 to 4                                       ' get five bytes (4 raw + cs)
      repeat nbit from 7 to 0                                    ' MSBFIRST
        phsa := 0                                                ' clear timer 
        waitpeq(mask, mask, 0)                                   ' wait for start pulse  
        waitpne(mask, mask, 0)                                   ' wait for end of pulse   
        if (phsa => bit1)                                        ' check bit timing
          raw.byte[idx] |= |< nbit                               '  set bit if 1

    repeat idx from 0 to 3                                       ' validate packet                                      
      cs -= raw.byte[idx]

    if (cs == 0)                                                 ' if good packet
      rhtresult := raw                                           ' transfer result to user
      badchecksum := false                                       ' clear error flag
    else
      badchecksum := true                                        ' set error flag
        
    ++count                                                      ' update reading count    
    waitcnt(t += secs)                                           ' let loop timer expire
    

dat { license }

{{

  Copyright (C) 2015 Jon McPhalen

  Terms of Use: MIT License

  Permission is hereby granted, free of charge, to any person obtaining a copy of this
  software and associated documentation files (the "Software"), to deal in the Software
  without restriction, including without limitation the rights to use, copy, modify,
  merge, publish, distribute, sublicense, and/or sell copies of the Software, and to
  permit persons to whom the Software is furnished to do so, subject to the following
  conditions:

  The above copyright notice and this permission notice shall be included in all copies
  or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
  INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
  PARTICULAR PURPOSE AND NON-INFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
  HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF
  CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE
  OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

}}